grammar javagrammar;

options
{
	superClass=BnfParser;
}

@header
{
package Grammar;

import java.util.Stack;
}

@lexer::header
{
package Grammar;
}

@parser::members
{
	public void bnf() throws RecognitionException
	{
		compilation_unit();
	}
}
@lexer::members {
    public Token nextToken() {
	while (true) {
		state.token = null;
		state.channel = Token.DEFAULT_CHANNEL;
		state.tokenStartCharIndex = input.index();
		state.tokenStartCharPositionInLine = input.getCharPositionInLine();
		state.tokenStartLine = input.getLine();
		state.text = null;
		if ( input.LA(1)==CharStream.EOF ) {
			return Token.EOF_TOKEN;
		}
		try {
			mTokens();
			if ( state.token==null ) {
				emit();
			}
			else if ( state.token==Token.SKIP_TOKEN ) {
				continue;
			}
			return state.token;
		}
		catch (RecognitionException re) {
				emit();
			return state.token;
		}
	}
}
}

compilation_unit :
		{Stack<String> stack = new Stack<String>(); trace.push(stack); stack.push("compilation_unit"); popLast = false;}
(({stack.push("id0"); popLast = true;} package_statement {popLast = false;}) )?(({stack.push("id1"); popLast = true;} import_statement {popLast = false;}) )*(({stack.push("id2"); popLast = true;} type_declaration {popLast = false;}) )*EOF
		{trace.pop();};

arglist :
		{Stack<String> stack = new Stack<String>(); trace.push(stack); stack.push("arglist"); popLast = false;}
({stack.push("id247"); popLast = true;} expression {popLast = false;}) (({stack.push("id248"); popLast = true;} ',' {popLast = false;}) ({stack.push("id249"); popLast = true;} expression {popLast = false;}) )*
		{trace.pop();};

expression :
		{Stack<String> stack = new Stack<String>(); trace.push(stack); stack.push("expression"); popLast = false;}
(({stack.push("id167"); popLast = true;} expressionPrefix {popLast = false;}) )?({stack.push("id168"); popLast = true;} expressionMain {popLast = false;}) (({stack.push("id169"); popLast = true;} '.' {popLast = false;}) ({stack.push("id170"); popLast = true;} expressionMain {popLast = false;}) )*(({stack.push("id171"); popLast = true;} expressionPost {popLast = false;}) )?
		{trace.pop();};

try_statement :
		{Stack<String> stack = new Stack<String>(); trace.push(stack); stack.push("try_statement"); popLast = false;}
({stack.push("id146"); popLast = true;} 'try' {popLast = false;}) ({stack.push("id147"); popLast = true;} statement_block {popLast = false;}) (({stack.push("id148"); popLast = true;} 'catch' {popLast = false;}) ({stack.push("id149"); popLast = true;} '(' {popLast = false;}) ({stack.push("id150"); popLast = true;} parameter {popLast = false;}) ({stack.push("id151"); popLast = true;} ')' {popLast = false;}) ({stack.push("id152"); popLast = true;} statement_block {popLast = false;}) )*(({stack.push("id153"); popLast = true;} 'finally' {popLast = false;}) ({stack.push("id154"); popLast = true;} statement_block {popLast = false;}) )?
		{trace.pop();};

if_statement :
		{Stack<String> stack = new Stack<String>(); trace.push(stack); stack.push("if_statement"); popLast = false;}
({stack.push("id116"); popLast = true;} 'if' {popLast = false;}) ({stack.push("id117"); popLast = true;} '(' {popLast = false;}) ({stack.push("id118"); popLast = true;} expression {popLast = false;}) ({stack.push("id119"); popLast = true;} ')' {popLast = false;}) ({stack.push("id120"); popLast = true;} statement_block {popLast = false;}) (({stack.push("id121"); popLast = true;} 'else' {popLast = false;}) ({stack.push("id122"); popLast = true;} statement {popLast = false;}) )?
		{trace.pop();};

interface_declaration :
		{Stack<String> stack = new Stack<String>(); trace.push(stack); stack.push("interface_declaration"); popLast = false;}
(({stack.push("id27"); popLast = true;} modifier {popLast = false;}) )*({stack.push("id28"); popLast = true;} 'interface' {popLast = false;}) ({stack.push("id29"); popLast = true;} IDENTIFIER {popLast = false;}) (({stack.push("id30"); popLast = true;} 'extends' {popLast = false;}) ({stack.push("id31"); popLast = true;} interface_name {popLast = false;}) (({stack.push("id32"); popLast = true;} ',' {popLast = false;}) ({stack.push("id33"); popLast = true;} interface_name {popLast = false;}) )*)?({stack.push("id34"); popLast = true;} '{' {popLast = false;}) (({stack.push("id35"); popLast = true;} field_declaration {popLast = false;}) )*({stack.push("id36"); popLast = true;} '}' {popLast = false;}) 
		{trace.pop();};

while_statement :
		{Stack<String> stack = new Stack<String>(); trace.push(stack); stack.push("while_statement"); popLast = false;}
({stack.push("id130"); popLast = true;} 'while' {popLast = false;}) ({stack.push("id131"); popLast = true;} '(' {popLast = false;}) ({stack.push("id132"); popLast = true;} expression {popLast = false;}) ({stack.push("id133"); popLast = true;} ')' {popLast = false;}) ({stack.push("id134"); popLast = true;} statement {popLast = false;}) 
		{trace.pop();};

interface_name :
		{Stack<String> stack = new Stack<String>(); trace.push(stack); stack.push("interface_name"); popLast = false;}
(({stack.push("id278"); popLast = true;} IDENTIFIER {popLast = false;}) ({stack.push("id279"); popLast = true;} '.' {popLast = false;}) )*({stack.push("id280"); popLast = true;} IDENTIFIER {popLast = false;}) 
		{trace.pop();};

class_name :
		{Stack<String> stack = new Stack<String>(); trace.push(stack); stack.push("class_name"); popLast = false;}
(({stack.push("id275"); popLast = true;} IDENTIFIER {popLast = false;}) ({stack.push("id276"); popLast = true;} '.' {popLast = false;}) )*({stack.push("id277"); popLast = true;} IDENTIFIER {popLast = false;}) 
		{trace.pop();};

type :
		{Stack<String> stack = new Stack<String>(); trace.push(stack); stack.push("type"); popLast = false;}
({stack.push("id250"); popLast = true;} type_specifier {popLast = false;}) (({stack.push("id251"); popLast = true;} '[' {popLast = false;}) ({stack.push("id252"); popLast = true;} ']' {popLast = false;}) )*
		{trace.pop();};

variable_declarator :
		{Stack<String> stack = new Stack<String>(); trace.push(stack); stack.push("variable_declarator"); popLast = false;}
({stack.push("id67"); popLast = true;} IDENTIFIER {popLast = false;}) (({stack.push("id68"); popLast = true;} '[' {popLast = false;}) ({stack.push("id69"); popLast = true;} ']' {popLast = false;}) )*(({stack.push("id70"); popLast = true;} '=' {popLast = false;}) ({stack.push("id71"); popLast = true;} variable_initializer {popLast = false;}) )?
		{trace.pop();};

variable_initializer :
		{Stack<String> stack = new Stack<String>(); trace.push(stack); stack.push("variable_initializer"); popLast = false;}
(({stack.push("id72"); popLast = true;} expression {popLast = false;}) |({stack.push("id73"); popLast = true;} '{' {popLast = false;}) (({stack.push("id74"); popLast = true;} variable_initializer {popLast = false;}) (({stack.push("id75"); popLast = true;} ',' {popLast = false;}) ({stack.push("id76"); popLast = true;} variable_initializer {popLast = false;}) )*(({stack.push("id77"); popLast = true;} ',' {popLast = false;}) )?)?({stack.push("id78"); popLast = true;} '}' {popLast = false;}) )
		{trace.pop();};

variable_declaration :
		{Stack<String> stack = new Stack<String>(); trace.push(stack); stack.push("variable_declaration"); popLast = false;}
(({stack.push("id61"); popLast = true;} modifier {popLast = false;}) )*({stack.push("id62"); popLast = true;} type {popLast = false;}) ({stack.push("id63"); popLast = true;} variable_declarator {popLast = false;}) (({stack.push("id64"); popLast = true;} ',' {popLast = false;}) ({stack.push("id65"); popLast = true;} variable_declarator {popLast = false;}) )*({stack.push("id66"); popLast = true;} ';' {popLast = false;}) 
		{trace.pop();};

parameter :
		{Stack<String> stack = new Stack<String>(); trace.push(stack); stack.push("parameter"); popLast = false;}
({stack.push("id84"); popLast = true;} type {popLast = false;}) ({stack.push("id85"); popLast = true;} IDENTIFIER {popLast = false;}) (({stack.push("id86"); popLast = true;} '[' {popLast = false;}) ({stack.push("id87"); popLast = true;} ']' {popLast = false;}) )*
		{trace.pop();};

constructor_declaration :
		{Stack<String> stack = new Stack<String>(); trace.push(stack); stack.push("constructor_declaration"); popLast = false;}
(({stack.push("id52"); popLast = true;} modifier {popLast = false;}) )*({stack.push("id53"); popLast = true;} IDENTIFIER {popLast = false;}) ({stack.push("id54"); popLast = true;} '(' {popLast = false;}) (({stack.push("id55"); popLast = true;} parameter_list {popLast = false;}) )?({stack.push("id56"); popLast = true;} ')' {popLast = false;}) ({stack.push("id57"); popLast = true;} statement_block {popLast = false;}) 
		{trace.pop();};

field_declaration :
		{Stack<String> stack = new Stack<String>(); trace.push(stack); stack.push("field_declaration"); popLast = false;}
((({stack.push("id37"); popLast = true;} method_declaration {popLast = false;}) |({stack.push("id38"); popLast = true;} constructor_declaration {popLast = false;}) |({stack.push("id39"); popLast = true;} variable_declaration {popLast = false;}) )|({stack.push("id40"); popLast = true;} static_initializer {popLast = false;}) |({stack.push("id41"); popLast = true;} ';' {popLast = false;}) )
		{trace.pop();};

expressionPrefix :
		{Stack<String> stack = new Stack<String>(); trace.push(stack); stack.push("expressionPrefix"); popLast = false;}
(({stack.push("id207"); popLast = true;} '-' {popLast = false;}) |({stack.push("id208"); popLast = true;} '++' {popLast = false;}) |({stack.push("id209"); popLast = true;} '--' {popLast = false;}) |({stack.push("id210"); popLast = true;} '!' {popLast = false;}) |({stack.push("id211"); popLast = true;} '~' {popLast = false;}) )
		{trace.pop();};

expressionPost :
		{Stack<String> stack = new Stack<String>(); trace.push(stack); stack.push("expressionPost"); popLast = false;}
(({stack.push("id212"); popLast = true;} '++' {popLast = false;}) |({stack.push("id213"); popLast = true;} '--' {popLast = false;}) |(({stack.push("id214"); popLast = true;} '=' {popLast = false;}) |({stack.push("id215"); popLast = true;} '+' {popLast = false;}) |({stack.push("id216"); popLast = true;} '+=' {popLast = false;}) |({stack.push("id217"); popLast = true;} '-' {popLast = false;}) |({stack.push("id218"); popLast = true;} '-=' {popLast = false;}) |({stack.push("id219"); popLast = true;} '*' {popLast = false;}) |({stack.push("id220"); popLast = true;} '*=' {popLast = false;}) |({stack.push("id221"); popLast = true;} '/' {popLast = false;}) |({stack.push("id222"); popLast = true;} '/=' {popLast = false;}) |({stack.push("id223"); popLast = true;} '%' {popLast = false;}) |({stack.push("id224"); popLast = true;} '%=' {popLast = false;}) |({stack.push("id225"); popLast = true;} '>' {popLast = false;}) |({stack.push("id226"); popLast = true;} '<' {popLast = false;}) |({stack.push("id227"); popLast = true;} '>=' {popLast = false;}) |({stack.push("id228"); popLast = true;} '<=' {popLast = false;}) |({stack.push("id229"); popLast = true;} '==' {popLast = false;}) |({stack.push("id230"); popLast = true;} '!=' {popLast = false;}) |({stack.push("id231"); popLast = true;} '&' {popLast = false;}) |({stack.push("id232"); popLast = true;} '&=' {popLast = false;}) |({stack.push("id233"); popLast = true;} '|' {popLast = false;}) |({stack.push("id234"); popLast = true;} '|=' {popLast = false;}) |({stack.push("id235"); popLast = true;} '^' {popLast = false;}) |({stack.push("id236"); popLast = true;} '^=' {popLast = false;}) |({stack.push("id237"); popLast = true;} '&&' {popLast = false;}) |({stack.push("id238"); popLast = true;} '||=' {popLast = false;}) |({stack.push("id239"); popLast = true;} '?' {popLast = false;}) ({stack.push("id240"); popLast = true;} expression {popLast = false;}) ({stack.push("id241"); popLast = true;} ':' {popLast = false;}) |({stack.push("id242"); popLast = true;} '>>=' {popLast = false;}) |({stack.push("id243"); popLast = true;} '<<' {popLast = false;}) |({stack.push("id244"); popLast = true;} '>>' {popLast = false;}) |({stack.push("id245"); popLast = true;} '>>>' {popLast = false;}) )({stack.push("id246"); popLast = true;} expression {popLast = false;}) )
		{trace.pop();};

for_statement :
		{Stack<String> stack = new Stack<String>(); trace.push(stack); stack.push("for_statement"); popLast = false;}
({stack.push("id135"); popLast = true;} 'for' {popLast = false;}) ({stack.push("id136"); popLast = true;} '(' {popLast = false;}) (({stack.push("id137"); popLast = true;} variable_declaration {popLast = false;}) |({stack.push("id138"); popLast = true;} expression {popLast = false;}) ({stack.push("id139"); popLast = true;} ';' {popLast = false;}) |({stack.push("id140"); popLast = true;} ';' {popLast = false;}) )(({stack.push("id141"); popLast = true;} expression {popLast = false;}) )?({stack.push("id142"); popLast = true;} ';' {popLast = false;}) (({stack.push("id143"); popLast = true;} expression {popLast = false;}) )?({stack.push("id144"); popLast = true;} ')' {popLast = false;}) ({stack.push("id145"); popLast = true;} statement {popLast = false;}) 
		{trace.pop();};

switch_statement :
		{Stack<String> stack = new Stack<String>(); trace.push(stack); stack.push("switch_statement"); popLast = false;}
({stack.push("id155"); popLast = true;} 'switch' {popLast = false;}) ({stack.push("id156"); popLast = true;} '(' {popLast = false;}) ({stack.push("id157"); popLast = true;} expression {popLast = false;}) ({stack.push("id158"); popLast = true;} ')' {popLast = false;}) ({stack.push("id159"); popLast = true;} '{' {popLast = false;}) ((({stack.push("id160"); popLast = true;} 'case' {popLast = false;}) ({stack.push("id161"); popLast = true;} expression {popLast = false;}) ({stack.push("id162"); popLast = true;} ':' {popLast = false;}) |({stack.push("id163"); popLast = true;} 'default' {popLast = false;}) ({stack.push("id164"); popLast = true;} ':' {popLast = false;}) |({stack.push("id165"); popLast = true;} statement {popLast = false;}) ))*({stack.push("id166"); popLast = true;} '}' {popLast = false;}) 
		{trace.pop();};

expressionMain :
		{Stack<String> stack = new Stack<String>(); trace.push(stack); stack.push("expressionMain"); popLast = false;}
(({stack.push("id172"); popLast = true;} 'null' {popLast = false;}) |({stack.push("id173"); popLast = true;} 'this' {popLast = false;}) |({stack.push("id174"); popLast = true;} 'super' {popLast = false;}) (({stack.push("id175"); popLast = true;} '(' {popLast = false;}) (({stack.push("id176"); popLast = true;} arglist {popLast = false;}) )?({stack.push("id177"); popLast = true;} ')' {popLast = false;}) )?|({stack.push("id178"); popLast = true;} IDENTIFIER {popLast = false;}) (({stack.push("id179"); popLast = true;} '(' {popLast = false;}) (({stack.push("id180"); popLast = true;} arglist {popLast = false;}) )?({stack.push("id181"); popLast = true;} ')' {popLast = false;}) )?(({stack.push("id182"); popLast = true;} '[' {popLast = false;}) ({stack.push("id183"); popLast = true;} expression {popLast = false;}) ({stack.push("id184"); popLast = true;} ']' {popLast = false;}) )*|({stack.push("id185"); popLast = true;} '(' {popLast = false;}) ({stack.push("id186"); popLast = true;} expression {popLast = false;}) ({stack.push("id187"); popLast = true;} ')' {popLast = false;}) |({stack.push("id188"); popLast = true;} 'true' {popLast = false;}) |({stack.push("id189"); popLast = true;} 'false' {popLast = false;}) |({stack.push("id190"); popLast = true;} integer_literal {popLast = false;}) |({stack.push("id191"); popLast = true;} FLOAT {popLast = false;}) |({stack.push("id192"); popLast = true;} STRING {popLast = false;}) |({stack.push("id193"); popLast = true;} 'new' {popLast = false;}) (({stack.push("id194"); popLast = true;} class_name {popLast = false;}) ({stack.push("id195"); popLast = true;} '(' {popLast = false;}) (({stack.push("id196"); popLast = true;} arglist {popLast = false;}) )?({stack.push("id197"); popLast = true;} ')' {popLast = false;}) |({stack.push("id198"); popLast = true;} type_specifier {popLast = false;}) ({stack.push("id199"); popLast = true;} '[' {popLast = false;}) (({stack.push("id200"); popLast = true;} expression {popLast = false;}) )?({stack.push("id201"); popLast = true;} ']' {popLast = false;}) (({stack.push("id202"); popLast = true;} '[' {popLast = false;}) ({stack.push("id203"); popLast = true;} ']' {popLast = false;}) )*|({stack.push("id204"); popLast = true;} '(' {popLast = false;}) ({stack.push("id205"); popLast = true;} expression {popLast = false;}) ({stack.push("id206"); popLast = true;} ')' {popLast = false;}) ))
		{trace.pop();};

classOrInterface_name :
		{Stack<String> stack = new Stack<String>(); trace.push(stack); stack.push("classOrInterface_name"); popLast = false;}
(({stack.push("id281"); popLast = true;} IDENTIFIER {popLast = false;}) ({stack.push("id282"); popLast = true;} '.' {popLast = false;}) )*({stack.push("id283"); popLast = true;} IDENTIFIER {popLast = false;}) 
		{trace.pop();};

static_initializer :
		{Stack<String> stack = new Stack<String>(); trace.push(stack); stack.push("static_initializer"); popLast = false;}
({stack.push("id79"); popLast = true;} 'static' {popLast = false;}) ({stack.push("id80"); popLast = true;} statement_block {popLast = false;}) 
		{trace.pop();};

import_statement :
		{Stack<String> stack = new Stack<String>(); trace.push(stack); stack.push("import_statement"); popLast = false;}
({stack.push("id6"); popLast = true;} 'import' {popLast = false;}) (({stack.push("id7"); popLast = true;} package_name {popLast = false;}) ({stack.push("id8"); popLast = true;} '.' {popLast = false;}) ({stack.push("id9"); popLast = true;} '*' {popLast = false;}) ({stack.push("id10"); popLast = true;} ';' {popLast = false;}) |({stack.push("id11"); popLast = true;} class_name {popLast = false;}) )({stack.push("id12"); popLast = true;} ';' {popLast = false;}) 
		{trace.pop();};

class_declaration :
		{Stack<String> stack = new Stack<String>(); trace.push(stack); stack.push("class_declaration"); popLast = false;}
(({stack.push("id15"); popLast = true;} modifier {popLast = false;}) )*({stack.push("id16"); popLast = true;} 'class' {popLast = false;}) ({stack.push("id17"); popLast = true;} IDENTIFIER {popLast = false;}) (({stack.push("id18"); popLast = true;} 'extends' {popLast = false;}) ({stack.push("id19"); popLast = true;} class_name {popLast = false;}) )?(({stack.push("id20"); popLast = true;} 'implements' {popLast = false;}) ({stack.push("id21"); popLast = true;} interface_name {popLast = false;}) (({stack.push("id22"); popLast = true;} ',' {popLast = false;}) ({stack.push("id23"); popLast = true;} interface_name {popLast = false;}) )*)?({stack.push("id24"); popLast = true;} '{' {popLast = false;}) (({stack.push("id25"); popLast = true;} field_declaration {popLast = false;}) )*({stack.push("id26"); popLast = true;} '}' {popLast = false;}) 
		{trace.pop();};

package_name :
		{Stack<String> stack = new Stack<String>(); trace.push(stack); stack.push("package_name"); popLast = false;}
(({stack.push("id272"); popLast = true;} IDENTIFIER {popLast = false;}) ({stack.push("id273"); popLast = true;} '.' {popLast = false;}) )*({stack.push("id274"); popLast = true;} IDENTIFIER {popLast = false;}) 
		{trace.pop();};

do_statement :
		{Stack<String> stack = new Stack<String>(); trace.push(stack); stack.push("do_statement"); popLast = false;}
({stack.push("id123"); popLast = true;} 'do' {popLast = false;}) ({stack.push("id124"); popLast = true;} statement_block {popLast = false;}) ({stack.push("id125"); popLast = true;} 'while' {popLast = false;}) ({stack.push("id126"); popLast = true;} '(' {popLast = false;}) ({stack.push("id127"); popLast = true;} expression {popLast = false;}) ({stack.push("id128"); popLast = true;} ')' {popLast = false;}) ({stack.push("id129"); popLast = true;} ';' {popLast = false;}) 
		{trace.pop();};

type_declaration :
		{Stack<String> stack = new Stack<String>(); trace.push(stack); stack.push("type_declaration"); popLast = false;}
(({stack.push("id13"); popLast = true;} class_declaration {popLast = false;}) |({stack.push("id14"); popLast = true;} interface_declaration {popLast = false;}) )
		{trace.pop();};

modifier :
		{Stack<String> stack = new Stack<String>(); trace.push(stack); stack.push("modifier"); popLast = false;}
(({stack.push("id262"); popLast = true;} 'public' {popLast = false;}) |({stack.push("id263"); popLast = true;} 'private' {popLast = false;}) |({stack.push("id264"); popLast = true;} 'protected' {popLast = false;}) |({stack.push("id265"); popLast = true;} 'static' {popLast = false;}) |({stack.push("id266"); popLast = true;} 'final' {popLast = false;}) |({stack.push("id267"); popLast = true;} 'native' {popLast = false;}) |({stack.push("id268"); popLast = true;} 'synchronized' {popLast = false;}) |({stack.push("id269"); popLast = true;} 'abstract' {popLast = false;}) |({stack.push("id270"); popLast = true;} 'threadsafe' {popLast = false;}) |({stack.push("id271"); popLast = true;} 'transient' {popLast = false;}) )
		{trace.pop();};

statement :
		{Stack<String> stack = new Stack<String>(); trace.push(stack); stack.push("statement"); popLast = false;}
(({stack.push("id88"); popLast = true;} variable_declaration {popLast = false;}) |({stack.push("id89"); popLast = true;} expression {popLast = false;}) ({stack.push("id90"); popLast = true;} ';' {popLast = false;}) |({stack.push("id91"); popLast = true;} statement_block {popLast = false;}) |({stack.push("id92"); popLast = true;} if_statement {popLast = false;}) |({stack.push("id93"); popLast = true;} do_statement {popLast = false;}) |({stack.push("id94"); popLast = true;} while_statement {popLast = false;}) |({stack.push("id95"); popLast = true;} for_statement {popLast = false;}) |({stack.push("id96"); popLast = true;} try_statement {popLast = false;}) |({stack.push("id97"); popLast = true;} switch_statement {popLast = false;}) |({stack.push("id98"); popLast = true;} 'synchronized' {popLast = false;}) ({stack.push("id99"); popLast = true;} '(' {popLast = false;}) ({stack.push("id100"); popLast = true;} expression {popLast = false;}) ({stack.push("id101"); popLast = true;} ')' {popLast = false;}) ({stack.push("id102"); popLast = true;} statement {popLast = false;}) |({stack.push("id103"); popLast = true;} 'return' {popLast = false;}) (({stack.push("id104"); popLast = true;} expression {popLast = false;}) )?({stack.push("id105"); popLast = true;} ';' {popLast = false;}) |({stack.push("id106"); popLast = true;} 'throw' {popLast = false;}) ({stack.push("id107"); popLast = true;} expression {popLast = false;}) ({stack.push("id108"); popLast = true;} ';' {popLast = false;}) |({stack.push("id109"); popLast = true;} 'break' {popLast = false;}) (({stack.push("id110"); popLast = true;} IDENTIFIER {popLast = false;}) )?({stack.push("id111"); popLast = true;} ';' {popLast = false;}) |({stack.push("id112"); popLast = true;} 'continue' {popLast = false;}) (({stack.push("id113"); popLast = true;} IDENTIFIER {popLast = false;}) )?({stack.push("id114"); popLast = true;} ';' {popLast = false;}) |({stack.push("id115"); popLast = true;} ';' {popLast = false;}) )
		{trace.pop();};

package_statement :
		{Stack<String> stack = new Stack<String>(); trace.push(stack); stack.push("package_statement"); popLast = false;}
({stack.push("id3"); popLast = true;} 'package' {popLast = false;}) ({stack.push("id4"); popLast = true;} package_name {popLast = false;}) ({stack.push("id5"); popLast = true;} ';' {popLast = false;}) 
		{trace.pop();};

statement_block :
		{Stack<String> stack = new Stack<String>(); trace.push(stack); stack.push("statement_block"); popLast = false;}
({stack.push("id58"); popLast = true;} '{' {popLast = false;}) (({stack.push("id59"); popLast = true;} statement {popLast = false;}) )*({stack.push("id60"); popLast = true;} '}' {popLast = false;}) 
		{trace.pop();};

method_declaration :
		{Stack<String> stack = new Stack<String>(); trace.push(stack); stack.push("method_declaration"); popLast = false;}
(({stack.push("id42"); popLast = true;} modifier {popLast = false;}) )*({stack.push("id43"); popLast = true;} type {popLast = false;}) ({stack.push("id44"); popLast = true;} IDENTIFIER {popLast = false;}) ({stack.push("id45"); popLast = true;} '(' {popLast = false;}) (({stack.push("id46"); popLast = true;} parameter_list {popLast = false;}) )?({stack.push("id47"); popLast = true;} ')' {popLast = false;}) (({stack.push("id48"); popLast = true;} '[' {popLast = false;}) ({stack.push("id49"); popLast = true;} ']' {popLast = false;}) )*(({stack.push("id50"); popLast = true;} statement_block {popLast = false;}) |({stack.push("id51"); popLast = true;} ';' {popLast = false;}) )
		{trace.pop();};

type_specifier :
		{Stack<String> stack = new Stack<String>(); trace.push(stack); stack.push("type_specifier"); popLast = false;}
(({stack.push("id253"); popLast = true;} 'boolean' {popLast = false;}) |({stack.push("id254"); popLast = true;} 'byte' {popLast = false;}) |({stack.push("id255"); popLast = true;} 'char' {popLast = false;}) |({stack.push("id256"); popLast = true;} 'short' {popLast = false;}) |({stack.push("id257"); popLast = true;} 'int' {popLast = false;}) |({stack.push("id258"); popLast = true;} 'float' {popLast = false;}) |({stack.push("id259"); popLast = true;} 'long' {popLast = false;}) |({stack.push("id260"); popLast = true;} 'double' {popLast = false;}) |({stack.push("id261"); popLast = true;} classOrInterface_name {popLast = false;}) )
		{trace.pop();};

integer_literal :
		{Stack<String> stack = new Stack<String>(); trace.push(stack); stack.push("integer_literal"); popLast = false;}
(({stack.push("id284"); popLast = true;} INT {popLast = false;}) (({stack.push("id285"); popLast = true;} 'l' {popLast = false;}) )?|({stack.push("id286"); popLast = true;} '0x' {popLast = false;}) ({stack.push("id287"); popLast = true;} HEX {popLast = false;}) (({stack.push("id288"); popLast = true;} HEX {popLast = false;}) )*)
		{trace.pop();};

parameter_list :
		{Stack<String> stack = new Stack<String>(); trace.push(stack); stack.push("parameter_list"); popLast = false;}
({stack.push("id81"); popLast = true;} parameter {popLast = false;}) (({stack.push("id82"); popLast = true;} ',' {popLast = false;}) ({stack.push("id83"); popLast = true;} parameter {popLast = false;}) )*
		{trace.pop();};

INT :	'0'..'9'+
    ;

FLOAT
    :   ('0'..'9')+ '.' ('0'..'9')* EXPONENT?
    |   '.' ('0'..'9')+ EXPONENT?
    |   ('0'..'9')+ EXPONENT
    ;

IDENTIFIER  :	('a'..'z'|'A'..'Z'|'_') ('a'..'z'|'A'..'Z'|'0'..'9'|'_')* ;

HEX
	:	HEX_DIGIT+;

STRING
:  '"' ( ESC_SEQ | ~('\\'|'"'|'\n') )* '"'
;

fragment
EXPONENT : ('e'|'E') ('+'|'-')? ('0'..'9')+ ;

fragment
HEX_DIGIT : ('0'..'9'|'a'..'f'|'A'..'F') ;

fragment
ESC_SEQ
    :   '\\' ('b'|'t'|'n'|'f'|'r'|'\"'|'\''|'\\')
    |   UNICODE_ESC
    |   OCTAL_ESC
    ;

fragment
OCTAL_ESC
    :   '\\' ('0'..'3') ('0'..'7') ('0'..'7')
    |   '\\' ('0'..'7') ('0'..'7')
    |   '\\' ('0'..'7')
    ;

fragment
UNICODE_ESC
    :   '\\' 'u' HEX_DIGIT HEX_DIGIT HEX_DIGIT HEX_DIGIT
    ;

COMMENT
    :   '//' ~('\n'|'\r')* '\r'? '\n' {$channel=HIDDEN;}
    |   '/*' ( options {greedy=false;} : . )* '*/' {$channel=HIDDEN;}
    ;

WS  :   ( ' '
        | '\t'
        | '\r'
        | '\n'
        ) {$channel=HIDDEN;}
    ;
